@@ -3,6 +3,7 @@ import sys, os
 from scrapy.commands import ScrapyCommand
 from scrapy.exceptions import UsageError
 
+
 class Command(ScrapyCommand):
 
     requires_project = True
@@ -15,7 +16,8 @@ class Command(ScrapyCommand):
         return "Edit spider"
 
     def long_desc(self):
-        return "Edit a spider using the editor defined in EDITOR setting"
+        return ("Edit a spider using the editor defined in the EDITOR environment"
+                " variable or else the EDITOR setting")
 
     def _err(self, msg):
         sys.stderr.write(msg + os.linesep)
